@@ -1,4 +1,5 @@
 import re
+from typing import Dict, Any, Tuple
 
 from ..abstract import Adapter
 
@@ -18,19 +19,19 @@ class ECSServiceCPUAlarmAdapter(Adapter):
         }
     """
 
-    def __init__(self, data, **kwargs):
+    def __init__(self, data: Dict[str, Any], **kwargs) -> None:
         self.cluster = kwargs.pop('cluster', None)
         self.service = kwargs.pop('service', None)
-        super(ECSServiceCPUAlarmAdapter, self).__init__(data, **kwargs)
+        super().__init__(data, **kwargs)
 
-    def get_AlarmName(self):
+    def get_AlarmName(self) -> str:
         if '<' in self.data['cpu']:
             direction = 'low'
         else:
             direction = 'high'
         return '{}-{}-{}'.format(self.cluster, self.service, direction)
 
-    def get_AlarmDescription(self):
+    def get_AlarmDescription(self) -> str:
         if '>' in self.data['cpu']:
             direction = 'up'
         else:
@@ -43,7 +44,7 @@ class ECSServiceCPUAlarmAdapter(Adapter):
             (int(self.data['periods']) * int(self.data['check_every_seconds']))
         )
 
-    def get_ComparisonOperator(self):
+    def get_ComparisonOperator(self) -> str:
         operator = '=='
         if '<=' in self.data['cpu']:
             operator = "LessThanOrEqualToThreshold"
@@ -55,11 +56,11 @@ class ECSServiceCPUAlarmAdapter(Adapter):
             operator = "GreaterThanThreshold"
         return operator
 
-    def get_Threshold(self):
+    def get_Threshold(self) -> float:
         return float(re.sub('[<>=]*', '', self.data['cpu']))
 
-    def convert(self):
-        data = {}
+    def convert(self) -> Tuple[Dict[str, Any], Dict[str, Any]]:
+        data: Dict[str, Any] = {}
         data['AlarmName'] = self.get_AlarmName()
         data['AlarmDescription'] = self.get_AlarmDescription()
         data['MetricName'] = 'CPUUtilization'
